﻿using Syncfusion.UI.Xaml.Diagram;
using Syncfusion.UI.Xaml.Diagram.Controls;
using Syncfusion.UI.Xaml.Diagram.Serializer;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Xml.Serialization;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Microsoft.Win32;
using Windows.Storage;
using System.Xml;
using Windows.Storage.Streams;
using System.Threading.Tasks;
using Windows.UI.Xaml.Media.Imaging;
using Windows.Graphics.Imaging;
using Windows.Graphics.Display;
using Windows.ApplicationModel;
using Windows.Storage.Pickers;
using System.ComponentModel;
using Windows.UI.Xaml.Shapes;
using System.Runtime.Serialization;
using Windows.UI;


// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409

namespace SfDiagram_Sample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            diagram.Nodes = new ObservableCollection<NodeViewModel>();

            NodeVM node = new NodeVM()
            {
                UnitHeight = 100,
                UnitWidth = 100,
                OffsetX = 100,
                OffsetY = 100,
                Shape = App.Current.Resources["Rectangle"],
                ShapeStyle = this.Resources["shapestyle"] as Style,
                Annotations = new ObservableCollection<IAnnotation>()
                {
                    new AnnotationEditorViewModel()
                    {
                        Content = "Annotation"
                    }
                }
            };

            (diagram.Nodes as ObservableCollection<NodeViewModel>).Add(node);

            diagram.Loaded += Diagram_Loaded;
        }

        private void Diagram_Loaded(object sender, RoutedEventArgs e)
        {
            var nodes = diagram.Nodes as ObservableCollection<NodeViewModel>;

            foreach (NodeVM node in nodes)
            {
                if (node.ShapeStyle != null)
                {
                    foreach (var setter in node.ShapeStyle.Setters.OfType<Setter>())
                    {
                        if (setter.Value is Brush)
                            (node as NodeVM).Fill = (setter.Value as SolidColorBrush).Color.ToString();
                    }
                }
            }        
        }
        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            StorageFolder storageFolder = ApplicationData.Current.LocalFolder;
            StorageFile createFile = await storageFolder.CreateFileAsync("serialize.xml", CreationCollisionOption.ReplaceExisting);
            using (Stream fileStream = await createFile.OpenStreamForWriteAsync())
            {
                diagram.Save(fileStream);
            }
        }

        private async void Button_Click_1(object sender, RoutedEventArgs e)
        {
            Windows.Storage.StorageFolder storageFolder = Windows.Storage.ApplicationData.Current.LocalFolder;
            Windows.Storage.StorageFile sampleFile = await storageFolder.GetFileAsync("serialize.xml");
            FileOpenPicker openPicker = new FileOpenPicker();
            openPicker.ViewMode = PickerViewMode.Thumbnail;
            openPicker.SuggestedStartLocation = PickerLocationId.PicturesLibrary;
            openPicker.FileTypeFilter.Add(".xml");

            StorageFile file = await openPicker.PickSingleFileAsync();

            using (Stream fileStream = await file.OpenStreamForReadAsync())
            {
                diagram.Load(fileStream);
            }

            var nodes = diagram.Nodes as ObservableCollection<NodeViewModel>;

            foreach (NodeVM node in nodes)
            {
                if (node is NodeVM)
                {
                    if (node.ShapeStyle == null)
                    {
                        Style style = new Style(typeof(Windows.UI.Xaml.Shapes.Path));
                        var color = GetSolidColorBrush(node.Fill.ToString()).Color;
                        var brush = new SolidColorBrush(color);
                        style.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.FillProperty, new SolidColorBrush(color)));
                        style.Setters.Add(new Setter(Windows.UI.Xaml.Shapes.Path.StretchProperty, Stretch.Fill));

                        node.ShapeStyle = style;
                    }
                }
            }
        }

        public SolidColorBrush GetSolidColorBrush(string hex)
        {
            hex = hex.Replace("#", string.Empty);
            byte a = (byte)(Convert.ToUInt32(hex.Substring(0, 2), 16));
            byte r = (byte)(Convert.ToUInt32(hex.Substring(2, 2), 16));
            byte g = (byte)(Convert.ToUInt32(hex.Substring(4, 2), 16));
            byte b = (byte)(Convert.ToUInt32(hex.Substring(6, 2), 16));
            SolidColorBrush myBrush = new SolidColorBrush(Windows.UI.Color.FromArgb(a, r, g, b));
            return myBrush;
        }
    }

    public class NodeVM : NodeViewModel, INotifyPropertyChanged
    {
        public NodeVM()
        {
            
        }

        private string _fill;
        [DataMember]
        public string Fill
        {
            get
            {
                return _fill;
            }
            set
            {

                _fill = value;
            }
        }        
    }
}
